/* Terraform - (C) 1997-2000 Robert Gasch (r.gasch@chello.nl)
 *  - http://212.187.12.197/RNG/terraform/
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include <stdio.h>
#include <time.h>
#include "GlobalSanityCheck.h"
#include "HeightFieldGenRandom.h"
#include "HeightFieldGenPerlin.h"
#include "HeightFieldGenSubdiv.h"
#include "HeightFieldGenSSynth.h"
#include "MathRandom.h"



/*
 *  initHF: intialize the HF data object that we have to fill
 */
int HeightFieldGenRandom::generate (HeightField *HF, int size)
{
	int		rc=0;
	MathRandom 	mRand (time(NULL));
	double		r;

	if (!HF)
		return -1;

	if (size<10)
		{
		SanityCheck::warning((size<10), "size<10 ... overriding", "HeightFieldGenRandom::generate()");
		size = 400;
		}

	r = mRand.rnd();
	if (r > 0.25)
		{
		HeightFieldGenSubdiv	*HFG = new HeightFieldGenSubdiv (HF);
		rc = HFG->generate ((int)(mRand.rnd(0,5)+1), size, 0.5, time(NULL));
		delete HFG;
		}
	else
	if (r < 0.125)
		{
		HeightFieldGenSSynth	*HFG = new HeightFieldGenSSynth (HF);
		rc = HFG->generate (size, 0.5, time(NULL), FALSE);
		delete HFG;
		}
	else
		{
		bool 	p1 = TRUE,
			p2 = (int)(mRand.rnd()*2),
			p3 = (int)(mRand.rnd()*2),
			p4 = (int)(mRand.rnd()*2);
		float	f1 = (float)(mRand.rnd()/2),
			f2 = (float)(mRand.rnd()/2),
			f3 = (float)(mRand.rnd()/2),
			f4 = (float)(mRand.rnd()/2),
			a1 = (float)(mRand.rnd()*5000+5000), 
			a2 = (float)(mRand.rnd()*5000+5000), 
			a3 = (float)(mRand.rnd()*5000+5000), 
			a4 = (float)(mRand.rnd()*5000+5000); 
		int	o1 = (int)(mRand.rnd()*10),
			o2 = (int)(mRand.rnd()*10),
			o3 = (int)(mRand.rnd()*10),
			o4 = (int)(mRand.rnd()*10),
			e1 = (int)(mRand.rnd()*22+1),
			e2 = (int)(mRand.rnd()*22+1),
			e3 = (int)(mRand.rnd()*22+1),
			e4 = (int)(mRand.rnd()*22+1);

		HeightFieldGenPerlin 	*HFG = new HeightFieldGenPerlin (HF);
		rc = HFG->generate (size, size, time(NULL), p1, p2, p3, p4,
				f1, f2, f3, f4, a1, a2, a3, a4, 
				o1, o2, o3, o4, e1, e2, e3, e4);
		delete HFG;
		}

	return rc;
}
